#include <ESP8266WiFi.h>
#include <OneWire.h>

OneWire  ds(4);

const char* ssid     = "<<wifi ssid>>";                     // wifi ssid
const char* password = "<<wifi password>>";                 // wifi password
String auth          = "<<authentication>>";                // Authentication credentials Create a string from <email_address>:<API_Password> and encode it base64
                                                            // The sample: 
                                                            //    String auth = "dXNlcjpwYXNzd29yZA==" 
                                                            // is the encoding for "user:password"

const char* host      = "www.ic2pro.com";
const int   httpPort  = 80;
String devId          = "111-222-444";                      // Device ID. CREATE YOUR OWN GUID; Use this http://www.guidgenerator.com/

String getTemperature()
{
  byte type_s;
  byte data[12];
  byte addr[8];
  float celsius;
  String scelsius;

  if ( !ds.search(addr)) {                                            // find chip
    if ( !ds.search(addr)) {
      ds.reset_search();
      delay(250);
      return "ERR1";
    }
  }

  if (OneWire::crc8(addr, 7) != addr[7]) {                            // check CRC
    return "ERR2";
  }

  switch (addr[0]) {                                                  // test if it's a proper chip
    case 0x10: type_s = 1; break;
    case 0x28: type_s = 0; break;
    case 0x22: type_s = 0; break;
    default:
      Serial.println("Device is not a DS18x20 family device.");       // Not a proper chip 
      return "ERR3";
  }

  ds.reset();
  ds.select(addr);
  ds.write(0x44, 1);                                                  // initiate conversion

  delay(1000); // maybe 750ms is enough, maybe not

  ds.reset();
  ds.select(addr);
  ds.write(0xBE);

  for (byte i = 0; i < 9; i++) {                                     // read data
    data[i] = ds.read();
  }

  
  int16_t raw = (data[1] << 8) | data[0];                            // Convert the data to temperature
  if (type_s) {
    raw = raw << 3;
    if (data[7] == 0x10) {
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    if (cfg == 0x00) raw = raw & ~7;                                // based on the resolution make the conversion 
    else if (cfg == 0x20) raw = raw & ~3;
    else if (cfg == 0x40) raw = raw & ~1;
  }
  celsius = (float)raw / 16.0;
  scelsius = String(celsius);
  return scelsius;
}

void sendTemperature() {
  Serial.print("connecting to ");
  Serial.println(host);
  
  // Use WiFiClient class to create TCP connections
  WiFiClient client;
  if (!client.connect(host, httpPort)) {
    Serial.println("connection failed");
    return;
  }
  
  // We now create a URI for the request
  String url = "/Wire/connector/set?id=" + devId + "&TEMPERATURE="+getTemperature();
  
  Serial.print("Requesting URL: ");
  Serial.println(url);
  
  // This will send the request to the server
  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Authorization: Basic " + auth + "\r\n" + 
               "Host: " + host + "\r\n" + 
               "Connection: close\r\n\r\n");
  delay(10);
  
  // Read all the lines of the reply from server and print them to Serial
  while(client.available()){
    String line = client.readStringUntil('\r');
    Serial.print(line);
  }
  
  Serial.println();
  Serial.println("closing connection");
}

void setup() {
  Serial.begin(115200);
  delay(10);

  // We start by connecting to a WiFi network

  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  
  WiFi.begin(ssid, password);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void loop() {
  sendTemperature();
  delay(2000);
}

